import type HasFieldData from '../Interfaces/HasFieldData';
import Field from './Field';
import Validations from '../Helpers/Validations';
import { fieldStore } from '$lib/Stores/FieldStore';

/**
 * MutipleField class.
 * 
 * @since 1.0.0
 */
export default class MultipleField extends Field {
    /**
     * Initializing field.
     * 
     * @param fieldData  Field data.
     * @param formValues Form values.
     * 
     * @since 1.0.0
     */
    public constructor(fieldData: HasFieldData, formValues: {} = {}) {        
        super(fieldData, formValues);

        if(formValues === undefined || formValues[this.id] === undefined) {
            this.formValues[this.id] = [];
        }

        if( this.valueDefault !== undefined && this.getValues() === undefined ) {
            this.formValues[this.id] = this.valueDefault;
        }
    }

    /**
     * Get values of the mutliple field.
     * 
     * @returns All values of the field.
     * 
     * @since 1.0.0
     */
    public getValues(): any[] {
        return this.formValues[this.id];
    }

    /**
    * Doing form things and get value of the field.
    * 
    * @returns Value of the field.
    * 
    * @since 1.0.0
    */
    public getValue(index: string): any {
        return this.formValues[this.id][index];
    }

    /**
     * Set value of the field.
     * 
     * @param value Value to set.
     * 
     * @since 1.0.0
     */
    public setValue(value: any, index: string): void {
        value = this.replace(value); // Auto replacements e.g. for points replace with comma.

        if( this.formValues[this.id][index] === value ) {
            return value;
        }

        this.formValues[this.id][index] = value;
        this.validate();
        this.updateAssets();
        
        fieldStore.set(this);

        return value;
    }

    /**
     * Set values of the field.
     * 
     * @param values All values of the field.
     * 
     * @since 1.0.0
     */
    public setValues(values: any[]) {
        if( this.formValues[this.id] === values ) {
            return values;
        }

        this.formValues[this.id] = values;
        this.validate();
		this.updateAssets();

        fieldStore.set(this);

        return values;
    }

    /**
     * Add new value to the field.
     * 
     * @param value New value.
     * 
     * @since 1.0.0
     */
    public addValue(value: any = '') {
        this.formValues[this.id].push(value);
    }

    /**
     * Remove value from the field.
     * 
     * @param index Index of the value to remove.
     * 
     * @since 1.0.0
     */
    public removeValue(index: string) {
        this.formValues[this.id].splice(index, 1);
    }

    /**
     * Validate field values and set errors.
     * 
     * @returns True if field is valid.
     * 
     * @since 1.0.0
     */
    public validate(): boolean {
        let validations: Validations;
        let errors: string[] = [];

        this.getValues().forEach( (value) => {  
            validations = new Validations(this.validations);         
            validations.validate(value);
            if(validations.getErrors().length > 0) {
                errors.concat(validations.getErrors());
            }
        });

        this.isValidated = true;
        this.errors = errors;
        return this.errors.length === 0;
    }
}