import type HasFormData from '../Interfaces/HasFormData';
import CSSClasses from '../Helpers/CSSClasses';
import Fieldset from './Fieldset';
import type HasFieldsetData from '../Interfaces/HasFieldsetData';
import type HasNavigationData from '../Interfaces/HasNavigationData';
import type Field from '../Fields/Field';
import Events from '../Actions/Events';
import type Trigger from '$lib/Types/Trigger';

/**
 * Class Form.
 * 
 * @since 1.0.0
 */
export default class Form {
    readonly id: string;
    readonly fieldsets: Fieldset[] = [];
    readonly classes: CSSClasses;
    readonly navigationData: HasNavigationData;
    readonly events: Events;

    private formValues: {} = {};
    private currentFieldsetId: string = '';

    /**
     * Initializing form data.
     * 
     * @param formData Formdata from JSON file.
     * 
     * @since 1.0.0
     */
    public constructor(formData: HasFormData, formValues: {} = {}) {
        this.id = formData.id;
        this.currentFieldsetId = formData.fieldsets[0].id;
        this.formValues = formValues;

        let fieldset: Fieldset;
        formData.fieldsets.forEach((fieldsetData: HasFieldsetData) => {
            fieldset = new Fieldset(fieldsetData, formValues);
            this.fieldsets.push(fieldset);
        });

        this.classes = new CSSClasses(formData.classes);
        this.navigationData = formData.navigation;
        this.events = new Events(formData.events, this.formValues);
    }

    /**
     * Execute triggers.
     * 
     * @return boolean True if all events were executed successfully.
     * 
     * @since 1.0.0
     */
    public executeEvents(trigger: Trigger): boolean {
        if (!this.events.exists()) {
            return true;
        }
        return this.events.execute(trigger);
    }

    /**
     * Execute callbacks.
     * 
     * @param trigger Trigger to execute.
     * 
     * @since 1.0.0
     */
    public executeFieldCallbacks(trigger: Trigger): void {
        this.fieldsets.forEach((fieldset: Fieldset) => {
            fieldset.fields.forEach((field: Field) => {
                if (field.valueCallback === undefined) {
                    return;
                }

                if (field.valueCallback.trigger !== trigger) {
                    return;
                }

                this.formValues[field.id] = field.valueCallback.execute();
            });
        });
    }

    /**
     * Set current fieldset.
     * 
     * @param fieldsetId Id of fieldset.
     * 
     * @since 1.0.0
     */
    public setCurrentFieldset(fieldsetId: string): void {
        if (!this.fieldsetExists(fieldsetId)) {
            console.error('Fieldset with id "' + fieldsetId + '" does not exist.');
            return;
        }

        this.currentFieldsetId = fieldsetId;
    }

    /**
     * Check if fieldset exists.
     * 
     * @param fieldsetId Fieldset id.
     * 
     * @returns boolean True if fieldset exists, false if not.
     * 
     * @since 1.0.0 
     */
    public fieldsetExists(fieldsetId: string): boolean {
        return this.fieldsets.find((fieldset: Fieldset) => {
            return fieldset.id === fieldsetId;
        }) !== undefined;
    }

    /**
     * Get current fieldset id.
     * 
     * @returns string Current fieldset id.
     * 
     * @since 1.0.0
     */
    public getCurrentFieldsetId(): string {
        return this.currentFieldsetId;
    }

    /**
     * Get current fieldset.
     * 
     * @returns {Fieldset} Fieldset object.
     * 
     * @since 1.0.0
     */
    public getCurrentFieldset(): Fieldset {
        return this.fieldsets.find((fieldset: Fieldset) => {
            return fieldset.id === this.currentFieldsetId;
        });
    }

    /**
     * Get value by field id.
     * 
     * @param fieldId Field id.
     * 
     * @returns {any} Value of field.
     * 
     * @since 1.0.0
     */
    public getValue(fieldId: string): any {
        return this.formValues[fieldId];
    }

    /**
     * Set value by field id.
     * 
     * @param fieldId Field id.
     * @param value Value to set.
     * 
     * @since 1.0.0
     */
    public setValue(fieldId: string, value: any): void {
        this.formValues[fieldId] = value;
    }

    /**
     * Get form values.
     * 
     * @returns [] Form values.
     * 
     * @since 1.0.0
     */
    public getValues(): {} {
        return this.formValues;
    }

    /**
     * Has navigation data (CSS etc).
     * 
     * @returns Boolean True if form has navigation data, false if not.
     * 
     * @since 1.0.0
     */
    public hasNavigationData(): boolean {
        return this.navigationData !== undefined;
    }
}