import type Field from '../Fields/Field';
import type HasValidationData from '../Interfaces/HasValidationData';
import ValidationMedhods from './ValidationMedhods';

/**
 * Validations class.
 * 
 * @since 1.0.0
 */
export default class Validations {
    private errors: string[] = [];
    private validated: boolean = false;

    /**
     * Constructor.
     * 
     * @param value      A value which have to be validated.
     * @param validation Validation rules array.
     * @param required   Is value required or not.
     * 
     * @since 1.0.0
     */
    constructor(
        private validations: HasValidationData[]
    ) {}
       
    /**
     * Doing check for given values.
     * 
     * @since 1.0.0
     */
    public validate(value) : string[] {
        this.errors = [];

        // Running each validation
        this.validations?.forEach( validation => {
            let valueAsNumber = ( (validation.value as unknown) as number );

            // Assigning Validation functions
            switch( validation.type ) {
                case 'string':
                    if ( ! ValidationMedhods.string( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'letters':
                    if ( ! ValidationMedhods.letters( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'int':
                    if ( ! ValidationMedhods.int( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'number':
                    if ( ! ValidationMedhods.number( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'email':
                    if ( ! ValidationMedhods.email( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'min':
                    if ( ! ValidationMedhods.min( value, valueAsNumber ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'max':
                    if ( ! ValidationMedhods.max( value, valueAsNumber ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'minLength':
                    if ( ! ValidationMedhods.minLength( value, valueAsNumber ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'maxLength':
                    if ( ! ValidationMedhods.maxLength( value, valueAsNumber ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'empty':
                    if ( ValidationMedhods.empty( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;                    
                case 'inArray':
                    if ( ! ValidationMedhods.inArray( value, validation.values ) ) {
                        this.errors.push( validation.error );
                    }
                    break;
                case 'isChecked':
                    if ( ! ValidationMedhods.isChecked( value ) ) {
                        this.errors.push( validation.error );
                    }
                    break;                    
                default:
                    this.errors.push( 'Validations-Typ "' + validation.type + '" existiert nicht."' );
                    break;
            }
        });

        this.validated = true;

        return this.errors;
    }

    /**
     * Are there errors after validation?
     * 
     * @returns True on errors, false if not.
     * 
     * @since 1.0.0
     */
    public isError() : boolean {
        return this.errors.length > 0 ? true : false;
    }

    /**
     * Get errors after validation.
     * 
     * @returns All errors.
     * 
     * @since 1.0.0
     */
    public getErrors() : string[] {
        return this.errors;
    }

    /**
     * Is validation done?
     * 
     * @returns True if validated, false if not.
     * 
     * @since 1.0.0
     */
    public isValidated() : boolean {
        return this.validated;
    }

}